# Generated by roxygen2: do not edit by hand

S3method(predict,lm2)
S3method(print,desc_var)
S3method(print,lm2)
S3method(print,t.test2)
S3method(print,table2)
S3method(summary,lm2)
S3method(t,test2)

export("var_labels<-")
export(clear)
export(clear_stimulus_cache)
export(convert_to_sql)
export(desc_var)
export(format_pvalue)
export(interprobe)
export(list2)
export(lm2)
export(lm2_notes)
export(message2)
export(plot_cdf)
export(plot_density)
export(plot_freq)
export(plot_gam)
export(plot_means)
export(resize_images)
export(scatter.gam)
export(stimulus.beeswarm)
export(stimulus.plot)
export(t.test2)
export(table2)
export(text2)
export(twolines)
export(var_labels)
import(lmerTest)
importFrom(beeswarm,beeswarm)
importFrom(digest,digest)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(grDevices,recordPlot)
importFrom(grDevices,replayPlot)
importFrom(grDevices,svg)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,rect)
importFrom(graphics,segments)
importFrom(graphics,strheight)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(lmtest,coeftest)
importFrom(mgcv,gam)
importFrom(sandwich,vcovHC)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,ave)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,drop.terms)
importFrom(stats,ecdf)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,ks.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,predict)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,t.test)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
